@@ -7,10 +7,12 @@ import threading
 import traceback
 import socket
 import struct
+import select
 import sys
 import os
 import atexit
 import code
+import signal
 
 VERBOSE = True
 
@@ -38,7 +40,7 @@ def cry(message):
     """
     if VERBOSE:
         try:
-            print(message, file=_STDERR)
+            print("Manhole: "+message, file=_STDERR)
         except: #pylint: disable=W0702
             pass
 
@@ -57,10 +59,9 @@ class Manhole(threading.Thread):
     Thread that runs the infamous "Manhole".
     """
 
-    def __init__(self, poll_interval):
+    def __init__(self):
         super(Manhole, self).__init__()
         self.daemon = True
-        self.poll_interval = poll_interval
         self.name = "Manhole"
 
     @staticmethod
@@ -204,23 +205,37 @@ def _patch_os_fork_functions():
         _ORIGINAL_OS_FORKPTY, os.forkpty = os.forkpty, _patched_forkpty
     cry("Patched %s and %s." % (_ORIGINAL_OS_FORK, _ORIGINAL_OS_FORKPTY))
 
-def install(poll_interval=5, verbose=True):
+def _activate_on_signal(_signum, _frame):
+    assert _INST
+    _INST.start()
+
+def install(verbose=True, patch_fork=True, activate_on=None):
     global _STDERR, _INST, VERBOSE #pylint: disable=W0603
     with _INST_LOCK:
         VERBOSE = verbose
         _STDERR = sys.__stderr__
         if not _INST:
-            _INST = Manhole(poll_interval)
-            _INST.start()
+            _INST = Manhole()
+            if activate_on is None:
+                _INST.start()
+            else:
+                signal.signal(
+                    getattr(signal, 'SIG'+activate_on) if isinstance(activate_on, basestring) else activate_on,
+                    _activate_on_signal
+                )
         atexit.register(_remove_manhole_uds)
-        _patch_os_fork_functions()
+        if patch_fork:
+            if activate_on is None:
+                _patch_os_fork_functions()
+            else:
+                cry("Not patching os.fork and os.forkpty. Activation is done by signal %s" % activate_on)
 
 def reinstall():
     global _INST #pylint: disable=W0603
     assert _INST
     with _INST_LOCK:
         if not _INST.is_alive():
-            _INST = Manhole(_INST.poll_interval)
+            _INST = Manhole()
             _INST.start()
 
 def dump_stacktraces():
